const COLORS = ["", "red", "orange", "green", "blue", "violet"];

function SimpleMailAddress(id, name, email, group, favorite) {
  this.id = id;
  this.name = name;
  this.email = email;
  this.group = group;
  this.favorite = favorite;

  this.getLogin = function() {
    return /(.*)@/.exec(this.email)[1];
  }
  this.getHost = function() {
    return /@(.*)/.exec(this.email)[1];
  }
  this.toString = function() {
    return this.name + (this.name && this.email ? " " : "") +
           (this.email ? "<" + this.email + ">" : "");
  }
  this.toHtml = function() {
    return this.name + (this.name && this.email ? " " : "") +
           (this.email ? "&lt;<a href='mailto:" + this.email + "'>" +
                         this.email + "</a>&gt;" : "");
  }
}

SimpleMailAddress.parse = function(text) {
  if (!text) return;

  var name, email;
  text = text.replace(/[^'"\s<>]+@[^'"\s<>]+/, function($0) {
    email = $0;
    return "";
  });
  name = text.replace(/^['"\s<>]*|['"\s<>]*$/g, "");
  if (name || email) {
    return new SimpleMailAddress(null, name, email);
  }
}

SimpleMailAddress.parseList = function(text) {
  var addresses = new Array();
  if (text) {
    var list = ("" + text).split(/[,;]/);
    for(var i in list) {
      var address = SimpleMailAddress.parse(list[i]);
      if (address) addresses.push(address);
    }
  }
  return addresses;
}

SimpleMailAddress.create = function() {
  return new SimpleMailAddress(null, null, null, null);
}

function SimpleMailAccount(id, index, name, email, replyTo, refresh, deleteMode,
                   color, subject, signature, isImap, pop3auth,
                   pop3host, pop3port, pop3ssl, pop3login,
                   smtpHost, smtpPort, smtpSsl, smtpLogin) {
  var self = this;
  this.id = id;
  this.index = index;
  this.name = name;
  this.email = email;
  this.replyTo = replyTo;
  this.refresh = refresh;
  this.deleteMode = deleteMode;
  this.color = color;
  this.subject = subject;
  this.signature = signature;

  this.pop3 = {
    isImap: isImap,
    auth:   pop3auth,
    host:   pop3host,
    port:   pop3port,
    ssl:    pop3ssl,
    login:  pop3login
  }
  this.pop3.getPassword = function() {
    return SimpleMailPasswordManager.getPassword(self.id + "/pop3/");
  }
  this.pop3.setPassword = function(password) {
    return SimpleMailPasswordManager.setPassword(self.id + "/pop3/", password);
  }
  this.smtp = {
    host:  smtpHost,
    port:  smtpPort,
    ssl:   smtpSsl,
    login: smtpLogin
  }
  this.smtp.getPassword = function() {
    return SimpleMailPasswordManager.getPassword(self.id + "/smtp/");
  }
  this.smtp.setPassword = function(password) {
    return SimpleMailPasswordManager.setPassword(self.id + "/smtp/", password);
  }
  this.toString = function() {
    return (this.name ? this.name + " " : "") + "<" + this.email + ">";
  }
}

SimpleMailAccount.create = function() {
  return new SimpleMailAccount(null,
                       null,
                       null,
                       null,
                       null,
                       60,
                       null,
                       null,
                       null,
                       null,
                       null,
                       0,
                       null, 
                       110,
                       null,
                       null,
                       null,
                       25,
                       null,
                       null);
}

SimpleMailAccount.ASK = "ask";
SimpleMailAccount.NEVER = "never";
SimpleMailAccount.ALWAYS = "always";
SimpleMailAccount.ON_RECEIVE = "onReceive";

function SimpleMailMessage(id, accountId, uid, from, to, cc, bcc, subject, date, size, charset,
                   read, deleted, attachmentsDir, attachmentsCount, html, folderId,
                   partial, template, color, returnReceiptTo, replyTo) {
  this.id = id;
  this.accountId = accountId;
  this.uid = uid;
  this.from = from;
  this.to = to || "";
  this.cc = cc || "";
  this.bcc = bcc || "";
  this.subject = subject;
  this.date = date;
  this.size = size;
  this.charset = charset;
  this.read = read;
  this.deleted = deleted;
  this.attachmentsDir = attachmentsDir || SimpleMailUtils.getUniqueId();
  this.attachmentsCount = attachmentsCount || 0;
  this.html = html;
  this.folderId = folderId;
  this.partial = partial;
  this.template = template;
  this.color = color;
  this.returnReceiptTo = returnReceiptTo;
  this.replyTo = replyTo;

  this.contentType;
  this.source;

  this.getSize = function() {
    if (this.size == null) {
      this.size = this.toString().length;
    }
    return this.size;
  }

  this.toString = function() {
    if (!this.source) {
      this.source = SimpleMailMessageEncoder.encode(this);
    }
    return this.source;
  }

  this.toHtml = function() {
    function mailList2html(value) {
      var addresses = SimpleMailAddress.parseList(value);
      var result = "";
      for(var i in addresses) {
        result += (result ? ", " : "") + addresses[i].toHtml();
      }
      return result;
    }
    var from = this.from ? SimpleMailAddress.parse(this.from).toHtml() : "";
    return "<span style='color: gray;'>" +
           "From: " + from + "<br>" +
           "To: " + mailList2html(this.to) + "<br>" + 
           (this.cc ? "Cc: " + mailList2html(this.cc) + "<br>" : "") +
           "Date: " + SimpleMailDate.toString(this.date) + "<br>" +
           "Subject: " + this.subject + "<br>" +
           "</span><br>" + this.html;
  }

  this.createReply = function(all) {
    var reply = new SimpleMailMessage();
    reply.accountId = this.accountId;
    reply.charset = this.charset;
    reply.to = this.replyTo || this.from;
    if (all) {
      if (this.to) reply.to += ", " + this.to;
      reply.cc = this.cc;
      reply.bcc = this.bcc;
    }
    reply.subject = this.subject.replace(/^(Re: )?/i, "Re: ");
    // Remove attachments
    reply.html = SimpleMailText.replaceURLs(this.html,
      function(text, url) {
        return SimpleMailFile.isLocalURL(url)
           ? "[Attachment: " + SimpleMailFile.getFileName(url) + "]" : text;
      });

    reply.html = "<br><br><span style='color: gray;'>" +
                 SimpleMailPrefs.getString("reply")
                        .replace(/{author}/gi, SimpleMailAddress.parse(this.from).toHtml())
                        .replace(/{date}/gi, SimpleMailDate.toString(this.date))
                        .replace(/{date-gmt}/gi, SimpleMailDate.toGMT(this.date))
                        .replace(/{date-locale}/gi, new Date(this.date).toLocaleString())
                        .replace(/{subject}/gi, this.subject) +
                 "</span><br>" +
                 "<blockquote style='border-left: 1px solid rgb(204, 204, 204);" +
                                    "margin: 0 0 0 0.8ex;" +
                                    "padding: 1ex 0 0 1ex;'>" + reply.html + "</blockquote>";
    return reply;
  }

  this.createForward = function() {
    var forward = new SimpleMailMessage();
    forward.accountId = this.accountId;
    forward.charset = this.charset;
    forward.subject = this.subject.replace(/^(Fwd?: )?/i, "Fwd: ");
    forward.html = "<br><br><hr>" + this.toHtml();
    return forward;
  }

  this.createReturnReceipt = function(recipient) {
    var receipt = new SimpleMailMessage();
    receipt.accountId = this.accountId;
    receipt.from = recipient;
    receipt.to = this.returnReceiptTo;
    receipt.charset = "utf-8";
    receipt.subject = "Rcpt: " + this.subject;
    receipt.html = SimpleMailBundle.getString("messageReceived");
    if (receipt.html != "Message received") {
      receipt.html = "Message received / " + receipt.html;
    }
    return receipt;
  }

  var fields = ["from", "to", "cc", "bcc", "subject", "charset",
                "html", "returnReceiptTo"];

  this.equals = function(message) {
    for(var i in fields) {
      var field = fields[i];
      if (this[field] != message[field]) return false;
    }
    return true;
  }

  this.copyTo = function(message) {
    for(var i in fields) {
      var field = fields[i];
      message[field] = this[field];
    }
  }

  this.recode = function(charset) {
    try {
      var message = new SimpleMailMessage();
      for(var i in fields) {
        var field = fields[i];
        message[field] = SimpleMailText.fromUnicode(this[field], this.charset);
        message[field] = SimpleMailText.toUnicode(message[field], charset);
      }
      message.charset = charset;
      // If no exceptions, update original message
      message.copyTo(this);
    }
    catch(e) {}
  }
}

SimpleMailMessage.parse = function(source) {
  return SimpleMailMessageEncoder.decode(source);
}

SimpleMailMessage.parseMailto = function(mailto) {
  function getUrlParameter(name) {
    var regexp = new RegExp("[?&]" + name + "=([^&]*)", "i");
    var result = regexp.exec(mailto);
    return result ? decodeURIComponent(result[1]) : null;
  }
  var message = new SimpleMailMessage();
  message.to = decodeURIComponent(/mailto:([^?&]*)/.exec(mailto)[1]);
  message.cc = getUrlParameter("cc");
  message.bcc = getUrlParameter("bcc");
  message.subject = getUrlParameter("subject");
  message.html = getUrlParameter("body");
  return message;
}

function SimpleMailFolder(id, parentId, name, open) {
  this.id = id;
  this.parentId = parentId;
  this.name = name;
  this.open = open;

  this.getName = function() {
    return SimpleMailBundle.getString(this.name) || this.name;
  }
}

SimpleMailFolder.DEFAULT_NAMES = ["inbox", "sent", "drafts", "trash"];
SimpleMailFolder.INBOX = 1;
SimpleMailFolder.SENT = 2;
SimpleMailFolder.DRAFTS = 3;
SimpleMailFolder.TRASH = 4;

function SimpleMailRule(id, filterId, property, not, condition, param) {
  this.id = id;
  this.filterId = filterId;
  this.property = property;
  this.not = not;
  this.condition = condition;
  this.param = param;

  function matchAll(string, patterns) {
    string = String.toLowerCase(string);
    for(var i = 0; i < patterns.length; i++) {
      var pattern = SimpleMailText.trim(patterns[i]).toLowerCase();
      if (string.indexOf(pattern) == -1) return false;
    }
    return true;
  }

  this.test = function(message, storage, mail) {
    var value = message[property];
    var result;

    switch(this.condition) {
      case "equals":
          result = value == this.param;
          break;
      case "contains":
          // "," separator means "or", "+" means "and"
          var any = this.param.split(/,/);
          for(var i = 0; !result && i < any.length; i++) {
            var all = any[i].split(/\+/);
            result |= matchAll(value, all);
          }
          break;
      case "inAddressBook":
          var address = SimpleMailAddress.parse(value);
          result = address && storage.getAddressByEmail(address.email);
          break;
      case "greaterThan":
          result = parseInt(value) > parseInt(this.param);
          break;
      default:
          SimpleMailUtils.error("Unknown conditon: " + this.condition);
          break;
    }
    return this.not ? !result : result;
  }
}

SimpleMailRule.getProperties = function() {
 return ["accountId", "from", "to", "subject", "html", "size", "source"];
}

SimpleMailRule.getConditions = function(property) {
  switch(property) {
    case "accountId":
        return ["equals"];
    case "from":
        return ["contains", "inAddressBook"];
    case "size":
        return ["greaterThan"];
    default:
        return ["contains"];
  }
}

SimpleMailRule.create = function() {
  return new SimpleMailRule(null, null, null, null, null, null);
}

function SimpleMailAction(id, filterId, name, param) {
  this.id = id;
  this.filterId = filterId;
  this.name = name;
  this.param = param;

  this.execute = function(message, storage, mail) {
    switch(this.name) {
      case "sound":
          SimpleMailSound.play(param);
          break
      case "color":
          message.color = param;
          break;
      case "folder":
          if (storage.getFolder(param)) {
            message.folderId = param;
          }
          break;
      case "delete":
          mail.delete(message);
          return;
      case "download":
          if (message.partial) {
            mail.getBody(message);
            return;
          }
          break;
      case "charset":
          message.recode(param);
          break;
      case "popup":
      case "counter":
          break; // Obsolete
      default:
          SimpleMailUtils.error("Unknown action: " + this.name);
          break;
    }
    return message;
  }
}

SimpleMailAction.getNames = function() {
  return [ "sound", "color", "folder", "delete", "download", "charset" ];
}

SimpleMailAction.create = function() {
  return new SimpleMailAction(null, null, null, null);
}

function SimpleMailFilter(id, index, name, type, rules, actions) {
  this.id = id;
  this.index = index;
  this.name = name;
  this.type = type || SimpleMailFilter.INBOX;
  this.rules = rules;
  this.actions = actions;

  this.getName = function() {
    return SimpleMailBundle.getString(this.name) || this.name;
  }

  this.execute = function(message, storage, mail) {
    for(var i = 0; i < this.rules.length; i++) {
      if (!this.rules[i].test(message, storage, mail)) return message;
    }
    for(var i = 0; message && i < this.actions.length; i++) {
      message = this.actions[i].execute(message, storage, mail);
    }
    return message;
  }
}

SimpleMailFilter.INBOX = 1;
SimpleMailFilter.SENT = 2;

SimpleMailFilter.create = function() {
  return new SimpleMailFilter(null, null, null, SimpleMailFilter.INBOX, new Array(), new Array());
}

function SimpleMailFilterManager(storage, mail) {
  this.storage = storage;
  this.mail = mail;

  var filters = storage.getFilters();

  this.execute = function(message, type) {
    for(var i = 0; message && i < filters.length; i++) {
      var filter = filters[i];
      if (type == filter.type) {
        message = filter.execute(message, this.storage, this.mail);
      }
    }
    return message;
  }
}
  